@@ -1,16 +1,16 @@
-# Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+# Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -34,7 +34,6 @@ from __future__ import annotations
 
 import os
 import re
-import warnings
 import weakref
 
 from abc import ABC, abstractmethod
@@ -73,7 +72,6 @@ except ImportError:
 from .constants import (
     CONN_ATTRS_DN,
     DEFAULT_CONFIGURATION,
-    DEPRECATED_TLS_VERSIONS,
     OPENSSL_CS_NAMES,
     TLS_CIPHER_SUITES,
     TLS_VERSIONS,
@@ -105,6 +103,7 @@ if OTEL_ENABLED:
     )
 
 from .optionfiles import read_option_files
+from .tls_ciphers import UNACCEPTABLE_TLS_CIPHERSUITES, UNACCEPTABLE_TLS_VERSIONS
 from .types import (
     BinaryProtocolType,
     DescriptionType,
@@ -129,7 +128,7 @@ TLS_VERSION_ERROR = (
     "TLS protocol version (should be one of {})."
 )
 
-TLS_VERSION_DEPRECATED_ERROR = (
+TLS_VERSION_UNACCEPTABLE_ERROR = (
     "The given tls_version: '{}' are no longer allowed (should be one of {})."
 )
 
@@ -203,7 +202,6 @@ class MySQLConnectionAbstract(ABC):
         self._force_ipv6: bool = False
         self._oci_config_file: Optional[str] = None
         self._oci_config_profile: Optional[str] = None
-        self._fido_callback: Optional[Union[str, Callable[[str], None]]] = None
         self._webauthn_callback: Optional[Union[str, Callable[[str], None]]] = None
         self._krb_service_principal: Optional[str] = None
 
@@ -340,7 +338,7 @@ class MySQLConnectionAbstract(ABC):
         # an older version.
         tls_versions.sort(reverse=True)  # type: ignore[union-attr]
         newer_tls_ver = tls_versions[0]
-        # translated_names[0] belongs to TLSv1, TLSv1.1 and TLSv1.2
+        # translated_names[0] are TLSv1.2 only
         # translated_names[1] are TLSv1.3 only
         translated_names: List[List[str]] = [[], []]
         iani_cipher_suites_names = {}
@@ -380,6 +378,18 @@ class MySQLConnectionAbstract(ABC):
                 "No valid cipher suite found in the 'tls_ciphersuites' list"
             )
 
+        # raise an error when using an unacceptable cipher
+        for cipher_as_ossl in translated_names[0]:
+            if cipher_as_ossl in UNACCEPTABLE_TLS_CIPHERSUITES["TLSv1.2"].values():
+                raise NotSupportedError(
+                    f"Cipher {cipher_as_ossl} when used with TLSv1.2 is unacceptable."
+                )
+        for cipher_as_ossl in translated_names[1]:
+            if cipher_as_ossl in UNACCEPTABLE_TLS_CIPHERSUITES["TLSv1.3"].values():
+                raise NotSupportedError(
+                    f"Cipher {cipher_as_ossl} when used with TLSv1.3 is unacceptable."
+                )
+
         self._ssl["tls_ciphersuites"] = [
             ":".join(translated_names[0]),
             ":".join(translated_names[1]),
@@ -441,13 +451,13 @@ class MySQLConnectionAbstract(ABC):
             )
 
         use_tls_versions = []
-        deprecated_tls_versions = []
+        unacceptable_tls_versions = []
         invalid_tls_versions = []
         for tls_ver in tls_versions:
             if tls_ver in TLS_VERSIONS:
                 use_tls_versions.append(tls_ver)
-            if tls_ver in DEPRECATED_TLS_VERSIONS:
-                deprecated_tls_versions.append(tls_ver)
+            if tls_ver in UNACCEPTABLE_TLS_VERSIONS:
+                unacceptable_tls_versions.append(tls_ver)
             else:
                 invalid_tls_versions.append(tls_ver)
 
@@ -456,12 +466,11 @@ class MySQLConnectionAbstract(ABC):
                 raise NotSupportedError(
                     TLS_VER_NO_SUPPORTED.format(tls_version, TLS_VERSIONS)
                 )
-            use_tls_versions.sort()
             self._ssl["tls_versions"] = use_tls_versions
-        elif deprecated_tls_versions:
+        elif unacceptable_tls_versions:
             raise NotSupportedError(
-                TLS_VERSION_DEPRECATED_ERROR.format(
-                    deprecated_tls_versions, TLS_VERSIONS
+                TLS_VERSION_UNACCEPTABLE_ERROR.format(
+                    unacceptable_tls_versions, TLS_VERSIONS
                 )
             )
         elif invalid_tls_versions:
@@ -731,13 +740,10 @@ class MySQLConnectionAbstract(ABC):
                 raise AttributeError(
                     "ssl_key and ssl_cert need to be both set, or neither"
                 )
-            if "tls_versions" in self._ssl and self._ssl["tls_versions"] is not None:
+            if self._ssl.get("tls_versions") is not None:
                 self._validate_tls_versions()
 
-            if (
-                "tls_ciphersuites" in self._ssl
-                and self._ssl["tls_ciphersuites"] is not None
-            ):
+            if self._ssl.get("tls_ciphersuites") is not None:
                 self._validate_tls_ciphersuites()
 
         if self._conn_attrs is None:
@@ -818,15 +824,6 @@ class MySQLConnectionAbstract(ABC):
                     KRB_SERVICE_PINCIPAL_ERROR.format(error="is incorrectly formatted")
                 )
 
-        if self._fido_callback:
-            warn_msg = (
-                "The `fido_callback` connection argument is deprecated and it will be "
-                "removed in a future release of MySQL Connector/Python. "
-                "Use `webauth_callback` instead"
-            )
-            warnings.warn(warn_msg, DeprecationWarning)
-            self._validate_callable("fido_callback", self._fido_callback, 1)
-
         if self._webauthn_callback:
             self._validate_callable("webauth_callback", self._webauthn_callback, 1)
 
