@@ -1,16 +1,16 @@
-# Copyright (c) 2023, Oracle and/or its affiliates.
+# Copyright (c) 2023, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -87,7 +87,14 @@ from ..types import (
     StatsPacketType,
     StrOrBytes,
 )
-from ..utils import get_platform, int1store, int4store, lc_int
+from ..utils import (
+    get_platform,
+    int1store,
+    int4store,
+    lc_int,
+    warn_ciphersuites_deprecated,
+    warn_tls_version_deprecated,
+)
 from .abstracts import MySQLConnectionAbstract, MySQLCursorAbstract, ServerInfo
 from .charsets import charsets
 from .cursor import (
@@ -153,6 +160,22 @@ class MySQLConnection(MySQLConnectionAbstract):
         # Post connection settings
         await self._post_connection()
 
+        # pylint: disable=protected-access
+        if (
+            not self._ssl_disabled
+            and hasattr(self._socket._writer, "get_extra_info")
+            and callable(self._socket._writer.get_extra_info)
+        ):
+            # Raise a deprecation warning if deprecated TLS version
+            # or cipher is being used.
+
+            # `get_extra_info("cipher")` returns a three-value tuple containing the name
+            # of the cipher being used, the version of the SSL protocol
+            # that defines its use, and the number of secret bits being used.
+            cipher, tls_version, _ = self._socket._writer.get_extra_info("cipher")
+            warn_tls_version_deprecated(tls_version)
+            warn_ciphersuites_deprecated(cipher, tls_version)
+
     def _add_default_conn_attrs(self) -> None:
         """Add the default connection attributes."""
         platform = get_platform()
@@ -318,7 +341,6 @@ class MySQLConnection(MySQLConnectionAbstract):
             oci_config_file=self._oci_config_file,
             oci_config_profile=self._oci_config_profile,
             webauthn_callback=self._webauthn_callback,
-            fido_callback=self._fido_callback,
         )
         self._handle_ok(ok_pkt)
 
