@@ -1,16 +1,16 @@
-# Copyright (c) 2009, 2023, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -29,6 +29,7 @@
 # mypy: disable-error-code="arg-type,operator,attr-defined,assignment"
 
 """Implementing communication with MySQL servers."""
+from __future__ import annotations
 
 import datetime
 import getpass
@@ -41,6 +42,7 @@ import warnings
 from decimal import Decimal
 from io import IOBase
 from typing import (
+    TYPE_CHECKING,
     Any,
     BinaryIO,
     Dict,
@@ -60,9 +62,9 @@ from .authentication import MySQLAuthenticator, get_auth_plugin
 from .constants import (
     ClientFlag,
     FieldType,
+    RefreshOption,
     ServerCmd,
     ServerFlag,
-    ShutdownType,
     flag_is_set,
 )
 from .conversion import MySQLConverter
@@ -97,7 +99,7 @@ from .opentelemetry.constants import OTEL_ENABLED
 from .opentelemetry.context_propagation import with_context_propagation
 from .protocol import MySQLProtocol
 from .types import (
-    ConnAttrsType,
+    BinaryProtocolType,
     DescriptionType,
     EofPacketType,
     HandShakeType,
@@ -106,9 +108,18 @@ from .types import (
     RowType,
     StatsPacketType,
     StrOrBytes,
-    SupportedMysqlBinaryProtocolTypes,
 )
-from .utils import get_platform, int1store, int4store, lc_int
+from .utils import (
+    get_platform,
+    int1store,
+    int4store,
+    lc_int,
+    warn_ciphersuites_deprecated,
+    warn_tls_version_deprecated,
+)
+
+if TYPE_CHECKING:
+    from .abstracts import CMySQLPrepStmt
 
 if OTEL_ENABLED:
     from .opentelemetry.instrumentation import end_span, record_exception_event
@@ -195,7 +206,7 @@ class MySQLConnection(MySQLConnectionAbstract):
 
     def _do_handshake(self) -> None:
         """Get the handshake from the MySQL server"""
-        packet = self._socket.recv()
+        packet = bytes(self._socket.recv())
         if packet[4] == 255:
             raise get_exception(packet)
 
@@ -246,7 +257,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         client_flags: int = 0,
         charset: int = 45,
         ssl_options: Optional[Dict[str, Optional[Union[str, bool, List[str]]]]] = None,
-        conn_attrs: Optional[ConnAttrsType] = None,
+        conn_attrs: Optional[Dict[str, str]] = None,
     ) -> bool:
         """Authenticate with the MySQL server
 
@@ -302,7 +313,6 @@ class MySQLConnection(MySQLConnectionAbstract):
             oci_config_file=self._oci_config_file,
             oci_config_profile=self._oci_config_profile,
             webauthn_callback=self._webauthn_callback,
-            fido_callback=self._fido_callback,
         )
         self._handle_ok(ok_pkt)
 
@@ -319,7 +329,7 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         Returns subclass of MySQLBaseSocket.
         """
-        conn: Optional[MySQLSocket] = None
+        conn = None
         if self._unix_socket and os.name == "posix":
             conn = MySQLUnixSocket(unix_socket=self.unix_socket)
         else:
@@ -375,21 +385,28 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         if (
             not self._ssl_disabled
-            and hasattr(self._socket.sock, "version")
-            and callable(self._socket.sock.version)
+            and hasattr(self._socket.sock, "cipher")
+            and callable(self._socket.sock.cipher)
         ):
-            # Raise a deprecation warning if TLSv1 or TLSv1.1 is being used
-            tls_version = self._socket.sock.version()
-            if tls_version in ("TLSv1", "TLSv1.1"):
-                warn_msg = (
-                    f"This connection is using {tls_version} which is now "
-                    "deprecated and will be removed in a future release of "
-                    "MySQL Connector/Python"
-                )
-                warnings.warn(warn_msg, DeprecationWarning)
+            # Raise a deprecation warning if deprecated TLS version
+            # or cipher is being used.
+
+            # `cipher()` returns a three-value tuple containing the name
+            # of the cipher being used, the version of the SSL protocol
+            # that defines its use, and the number of secret bits being used.
+            cipher, tls_version, _ = self._socket.sock.cipher()
+            warn_tls_version_deprecated(tls_version)
+            warn_ciphersuites_deprecated(cipher, tls_version)
 
     def shutdown(self) -> None:
-        """Shut down connection to MySQL Server."""
+        """Shut down connection to MySQL Server.
+
+        This method closes the socket. It raises no exceptions.
+
+        Unlike `disconnect()`, `shutdown()` closes the client connection without
+        attempting to send a QUIT command to the server first. Thus, it will not
+        block if the connection is disrupted for some reason such as network failure.
+        """
         if not self._socket:
             return
 
@@ -460,11 +477,11 @@ class MySQLConnection(MySQLConnectionAbstract):
         except AttributeError as err:
             raise OperationalError("MySQL Connection not available") from err
 
-        if not expect_response:
-            return None
-        return self._socket.recv()
+        return self._socket.recv() if expect_response else None
 
-    def _send_data(self, data_file: BinaryIO, send_empty_packet: bool = False) -> bytes:
+    def _send_data(
+        self, data_file: BinaryIO, send_empty_packet: bool = False
+    ) -> bytearray:
         """Send data to the MySQL server
 
         This method accepts a file-like object and sends its data
@@ -494,7 +511,7 @@ class MySQLConnection(MySQLConnectionAbstract):
             except AttributeError as err:
                 raise OperationalError("MySQL Connection not available") from err
 
-        return bytes(self._socket.recv())
+        return self._socket.recv()
 
     def _handle_server_status(self, flags: int) -> None:
         """Handle the server flags found in MySQL packets
@@ -641,6 +658,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         binary: bool = False,
         columns: Optional[List[DescriptionType]] = None,
         raw: Optional[bool] = None,
+        prep_stmt: Optional[CMySQLPrepStmt] = None,
     ) -> Tuple[Optional[RowType], Optional[EofPacketType]]:
         """Get the next rows returned by the MySQL server
 
@@ -662,7 +680,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         binary: bool = False,
         columns: Optional[List[DescriptionType]] = None,
         raw: Optional[bool] = None,
-        prep_stmt: Any = None,
+        prep_stmt: Optional[CMySQLPrepStmt] = None,
     ) -> Tuple[List[RowType], Optional[EofPacketType]]:
         """Get all rows returned by the MySQL server
 
@@ -678,7 +696,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         if not self.unread_result:
             raise InternalError("No result set available")
 
-        rows: Tuple[List[Tuple[Any, ...]], Optional[EofPacketType]] = ([], None)
+        rows: Tuple[List[Tuple], Optional[EofPacketType]] = ([], None)
         try:
             if binary:
                 charset = self.charset
@@ -907,19 +925,40 @@ class MySQLConnection(MySQLConnectionAbstract):
          RefreshOption = mysql.connector.RefreshOption
          refresh = RefreshOption.LOG | RefreshOption.THREADS
          cnx.cmd_refresh(refresh)
-
-        The result is a dictionary with the OK packet information.
-
-        Returns a dict()
         """
-        return self._handle_ok(self._send_cmd(ServerCmd.REFRESH, int4store(options)))
+        if not options & (
+            RefreshOption.GRANT
+            | RefreshOption.LOG
+            | RefreshOption.TABLES
+            | RefreshOption.HOST
+            | RefreshOption.STATUS
+            | RefreshOption.REPLICA
+        ):
+            raise ValueError("Invalid command REFRESH option")
+
+        if options & RefreshOption.GRANT:
+            res = self.cmd_query("FLUSH PRIVILEGES")
+        if options & RefreshOption.LOG:
+            res = self.cmd_query("FLUSH LOGS")
+        if options & RefreshOption.TABLES:
+            res = self.cmd_query("FLUSH TABLES")
+        if options & RefreshOption.HOST:
+            res = self.cmd_query("TRUNCATE TABLE performance_schema.host_cache")
+        if options & RefreshOption.STATUS:
+            res = self.cmd_query("FLUSH STATUS")
+        if options & RefreshOption.REPLICA:
+            res = self.cmd_query(
+                "RESET SLAVE" if self._server_version < (8, 0, 22) else "RESET REPLICA"
+            )
+
+        return res
 
     def cmd_quit(self) -> bytes:
         """Close the current connection with the server
 
-        This method sends the QUIT command to the MySQL server, closing the
-        current connection. Since the no response can be returned to the
-        client, cmd_quit() will return the packet it send.
+        This method sends the `QUIT` command to the MySQL server, closing the
+        current connection. Since there is no response from the MySQL server,
+        the packet that was sent is returned.
 
         Returns a str()
         """
@@ -927,26 +966,15 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         packet = self._protocol.make_command(ServerCmd.QUIT)
         self._socket.send(packet, 0, 0)
-        return bytes(packet)
+        return packet
 
-    def cmd_shutdown(self, shutdown_type: Optional[int] = None) -> EofPacketType:
+    def cmd_shutdown(self, shutdown_type: Optional[int] = None) -> None:
         """Shut down the MySQL Server
 
-        This method sends the SHUTDOWN command to the MySQL server and is only
-        possible if the current user has SUPER privileges. The result is a
-        dictionary containing the OK packet information.
-
-        Note: Most applications and scripts do not the SUPER privilege.
-
-        Returns a dict()
+        This method sends the SHUTDOWN command to the MySQL server.
+        The `shutdown_type` is not used, and it's kept for backward compatibility.
         """
-        if shutdown_type:
-            if not ShutdownType.get_info(shutdown_type):
-                raise InterfaceError("Invalid shutdown type")
-            atype = shutdown_type
-        else:
-            atype = ShutdownType.SHUTDOWN_DEFAULT
-        return self._handle_eof(self._send_cmd(ServerCmd.SHUTDOWN, int4store(atype)))
+        self.cmd_query("SHUTDOWN")
 
     def cmd_statistics(self) -> StatsPacketType:
         """Send the statistics command to the MySQL Server
@@ -968,12 +996,10 @@ class MySQLConnection(MySQLConnectionAbstract):
         This method send the PROCESS_KILL command to the server along with
         the process ID. The result is a dictionary with the OK packet
         information.
-
-        Returns a dict()
         """
-        return self._handle_ok(
-            self._send_cmd(ServerCmd.PROCESS_KILL, int4store(mysql_pid))
-        )
+        if not isinstance(mysql_pid, int):
+            raise ValueError("MySQL PID must be int")
+        return self.cmd_query(f"KILL {mysql_pid}")
 
     def cmd_debug(self) -> EofPacketType:
         """Send the DEBUG command
@@ -1060,7 +1086,6 @@ class MySQLConnection(MySQLConnectionAbstract):
             oci_config_file=self._oci_config_file,
             oci_config_profile=self._oci_config_profile,
             webauthn_callback=self._webauthn_callback,
-            fido_callback=self._fido_callback,
         )
 
         if not (self._client_flags & ClientFlag.CONNECT_WITH_DB) and database:
@@ -1186,7 +1211,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         buffered: Optional[bool] = None,
         raw: Optional[bool] = None,
         prepared: Optional[bool] = None,
-        cursor_class: Optional[Type[MySQLCursor]] = None,
+        cursor_class: Optional[Type[MySQLCursor]] = None,  # type: ignore[override]
         dictionary: Optional[bool] = None,
         named_tuple: Optional[bool] = None,
     ) -> MySQLCursor:
@@ -1269,7 +1294,7 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         self._execute_query("ROLLBACK")
 
-    def _execute_query(self, query: StrOrBytes) -> None:
+    def _execute_query(self, query: str) -> None:
         """Execute a query
 
         This method simply calls cmd_query() after checking for unread
@@ -1281,7 +1306,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         self.handle_unread_result()
         self.cmd_query(query)
 
-    def info_query(self, query: StrOrBytes) -> Optional[RowType]:
+    def info_query(self, query: str) -> Optional[RowType]:
         """Send a query which only returns 1 row"""
         cursor = self.cursor(buffered=True)
         cursor.execute(query)
@@ -1391,8 +1416,8 @@ class MySQLConnection(MySQLConnectionAbstract):
     def cmd_stmt_execute(
         self,
         statement_id: int,
-        data: Sequence[SupportedMysqlBinaryProtocolTypes] = (),
-        parameters: Sequence[Any] = (),
+        data: Sequence[BinaryProtocolType] = (),
+        parameters: Sequence = (),
         flags: int = 0,
     ) -> Union[OkPacketType, Tuple[int, List[DescriptionType], EofPacketType]]:
         """Execute a prepared MySQL statement"""
@@ -1439,7 +1464,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         result = self._handle_binary_result(packet)
         return result
 
-    def cmd_stmt_close(self, statement_id: int) -> None:
+    def cmd_stmt_close(self, statement_id: int) -> None:  # type: ignore[override]
         """Deallocate a prepared MySQL statement
 
         This method deallocates the prepared statement using the
@@ -1453,7 +1478,7 @@ class MySQLConnection(MySQLConnectionAbstract):
         )
 
     def cmd_stmt_send_long_data(
-        self, statement_id: int, param_id: int, data: BinaryIO
+        self, statement_id: int, param_id: int, data: BinaryIO  # type: ignore[override]
     ) -> int:
         """Send data for a column
 
@@ -1491,7 +1516,7 @@ class MySQLConnection(MySQLConnectionAbstract):
 
         return total_sent
 
-    def cmd_stmt_reset(self, statement_id: int) -> None:
+    def cmd_stmt_reset(self, statement_id: int) -> None:  # type: ignore[override]
         """Reset data for prepared statement sent as long data
 
         The result is a dictionary with OK packet information.
