@@ -1,16 +1,16 @@
-# Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+# Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -50,7 +50,7 @@ from typing import (
 
 from . import version
 from .abstracts import CMySQLPrepStmt, MySQLConnectionAbstract
-from .constants import ClientFlag, FieldFlag, ServerFlag, ShutdownType
+from .constants import ClientFlag, FieldFlag, FieldType, ServerFlag, ShutdownType
 from .conversion import MySQLConverter
 from .errors import (
     InterfaceError,
@@ -69,7 +69,11 @@ from .types import (
     StatsPacketType,
     StrOrBytes,
 )
-from .utils import import_object
+from .utils import (
+    import_object,
+    warn_ciphersuites_deprecated,
+    warn_tls_version_deprecated,
+)
 
 HAVE_CMYSQL = False
 
@@ -255,7 +259,7 @@ class CMySQLConnection(MySQLConnectionAbstract):
         # pylint: enable=c-extension-no-member
         if not self.isset_client_flag(ClientFlag.CONNECT_ARGS):
             self._conn_attrs = {}
-        fido_callback = self._webauthn_callback or self._fido_callback
+
         cnx_kwargs = {
             "host": self._host,
             "user": self._user,
@@ -274,10 +278,10 @@ class CMySQLConnection(MySQLConnectionAbstract):
             "load_data_local_dir": self._allow_local_infile_in_path,
             "oci_config_file": self._oci_config_file,
             "oci_config_profile": self._oci_config_profile,
-            "fido_callback": (
-                import_object(fido_callback)
-                if isinstance(fido_callback, str)
-                else fido_callback
+            "webauthn_callback": (
+                import_object(self._webauthn_callback)
+                if isinstance(self._webauthn_callback, str)
+                else self._webauthn_callback
             ),
         }
 
@@ -331,6 +335,20 @@ class CMySQLConnection(MySQLConnectionAbstract):
 
         self._do_handshake()
 
+        if (
+            not self._ssl_disabled
+            and hasattr(self._cmysql, "get_ssl_cipher")
+            and callable(self._cmysql.get_ssl_cipher)
+        ):
+            # Raise a deprecation warning if deprecated TLS version
+            # or cipher is being used.
+
+            # `get_ssl_cipher()` returns the name of the cipher being used.
+            cipher = self._cmysql.get_ssl_cipher()
+            for tls_version in set(self._ssl.get("tls_versions", [])):
+                warn_tls_version_deprecated(tls_version)
+                warn_ciphersuites_deprecated(cipher, tls_version)
+
     def close(self) -> None:
         """Disconnect from the MySQL server"""
         if self._span and self._span.is_recording():
@@ -455,18 +473,36 @@ class CMySQLConnection(MySQLConnectionAbstract):
                 # convert the values. This can be accomplished by setting
                 # the raw option to True.
                 self._cmysql.raw(True)
+
             row = fetch_row()
             while row:
+                row = list(row)
+
+                if not self._cmysql.raw() and not raw:
+                    # `not _cmysql.raw()` means the c-ext conversion layer will happen.
+                    # `not raw` means the caller wants conversion to happen.
+                    # For a VECTOR type, the c-ext conversion layer cannot return
+                    # an array.array type since such a type isn't part of the Python/C
+                    # API. Therefore, the c-ext will treat VECTOR types as if they
+                    # were BLOB types - be returned as `bytes` always.
+                    # Hence, a VECTOR type must be cast to an array.array type using the
+                    # built-in python conversion layer.
+                    # pylint: disable=protected-access
+                    for i, dsc in enumerate(self._columns):
+                        if dsc[1] == FieldType.VECTOR:
+                            row[i] = MySQLConverter._vector_to_python(row[i])
+
                 if not self._raw and self.converter:
-                    row = list(row)
                     for i, _ in enumerate(row):
                         if not raw:
                             row[i] = self.converter.to_python(self._columns[i], row[i])
-                    row = tuple(row)
-                rows.append(row)
+
+                rows.append(tuple(row))
                 counter += 1
+
                 if count and counter == count:
                     break
+
                 row = fetch_row()
             if not row:
                 _eof: Optional[CextEofPacketType] = self.fetch_eof_columns(prep_stmt)[
