@@ -1,16 +1,16 @@
-# Copyright (c) 2009, 2023, Oracle and/or its affiliates. All rights reserved.
+# Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -35,11 +35,16 @@ from typing import Any, Dict, List, Optional, Sequence, Tuple, Union, ValuesView
 
 from .charsets import MYSQL_CHARACTER_SETS, MYSQL_CHARACTER_SETS_57
 from .errors import ProgrammingError
+from .tls_ciphers import APPROVED_TLS_VERSIONS, DEPRECATED_TLS_VERSIONS
 
 NET_BUFFER_LENGTH: int = 8192
 MAX_MYSQL_TABLE_COLUMNS: int = 4096
-# Flag used to send the Query Attributes with 0 (or more) parameters.
 PARAMETER_COUNT_AVAILABLE: int = 8
+"""Flag used to send the Query Attributes with 0 (or more) parameters."""
+MYSQL_VECTOR_TYPE_CODE = "f"
+"""Expected `typecode` when decoding VECTOR values from
+MySQL (blob) to Python (array.array).
+"""
 
 DEFAULT_CONFIGURATION: Dict[str, Optional[Union[str, bool, int]]] = {
     "database": None,
@@ -90,7 +95,6 @@ DEFAULT_CONFIGURATION: Dict[str, Optional[Union[str, bool, int]]] = {
     "krb_service_principal": None,
     "oci_config_file": None,
     "oci_config_profile": None,
-    "fido_callback": None,
     "webauthn_callback": None,
     "kerberos_auth_mode": None,
     "init_command": None,
@@ -98,9 +102,9 @@ DEFAULT_CONFIGURATION: Dict[str, Optional[Union[str, bool, int]]] = {
 
 CNX_POOL_ARGS: Tuple[str, str, str] = ("pool_name", "pool_size", "pool_reset_session")
 
-TLS_VERSIONS: List[str] = ["TLSv1.2", "TLSv1.3"]
 
-DEPRECATED_TLS_VERSIONS: List[str] = ["TLSv1", "TLSv1.1"]
+TLS_VERSIONS: List[str] = APPROVED_TLS_VERSIONS + DEPRECATED_TLS_VERSIONS
+"""Accepted TLS versions. A warning is raised when using a deprecated version."""
 
 
 def flag_is_set(flag: int, flags: int) -> bool:
@@ -180,7 +184,41 @@ class _Flags(_Constants):
 
 
 class FieldType(_Constants):
-    """MySQL Field Types"""
+    """MySQL Field Types.
+
+    This class provides all supported MySQL field or data types. They can be useful
+    when dealing with raw data or defining your own converters. The field type is
+    stored with every cursor in the description for each column.
+
+    The `FieldType` class shouldn't be instantiated.
+
+    Examples:
+        The following example shows how to print the name of the data type for
+        each column in a result set.
+
+        ```
+        from __future__ import print_function
+        import mysql.connector
+        from mysql.connector import FieldType
+
+        cnx = mysql.connector.connect(user='scott', database='test')
+        cursor = cnx.cursor()
+
+        cursor.execute(
+        "SELECT DATE(NOW()) AS `c1`, TIME(NOW()) AS `c2`, "
+        "NOW() AS `c3`, 'a string' AS `c4`, 42 AS `c5`")
+        rows = cursor.fetchall()
+
+        for desc in cursor.description:
+            colname = desc[0]
+            coltype = desc[1]
+            print("Column {} has type {}".format(
+                colname, FieldType.get_info(coltype)))
+
+        cursor.close()
+        cnx.close()
+        ```
+    """
 
     prefix: str = "FIELD_TYPE_"
     DECIMAL: int = 0x00
@@ -200,6 +238,7 @@ class FieldType(_Constants):
     NEWDATE: int = 0x0E
     VARCHAR: int = 0x0F
     BIT: int = 0x10
+    VECTOR: int = 0xF2
     JSON: int = 0xF5
     NEWDECIMAL: int = 0xF6
     ENUM: int = 0xF7
@@ -213,34 +252,35 @@ class FieldType(_Constants):
     GEOMETRY: int = 0xFF
 
     desc: Dict[str, Tuple[int, str]] = {
-        "DECIMAL": (0x00, "DECIMAL"),
-        "TINY": (0x01, "TINY"),
-        "SHORT": (0x02, "SHORT"),
-        "LONG": (0x03, "LONG"),
-        "FLOAT": (0x04, "FLOAT"),
-        "DOUBLE": (0x05, "DOUBLE"),
-        "NULL": (0x06, "NULL"),
-        "TIMESTAMP": (0x07, "TIMESTAMP"),
-        "LONGLONG": (0x08, "LONGLONG"),
-        "INT24": (0x09, "INT24"),
-        "DATE": (0x0A, "DATE"),
-        "TIME": (0x0B, "TIME"),
-        "DATETIME": (0x0C, "DATETIME"),
-        "YEAR": (0x0D, "YEAR"),
-        "NEWDATE": (0x0E, "NEWDATE"),
-        "VARCHAR": (0x0F, "VARCHAR"),
-        "BIT": (0x10, "BIT"),
-        "JSON": (0xF5, "JSON"),
-        "NEWDECIMAL": (0xF6, "NEWDECIMAL"),
-        "ENUM": (0xF7, "ENUM"),
-        "SET": (0xF8, "SET"),
-        "TINY_BLOB": (0xF9, "TINY_BLOB"),
-        "MEDIUM_BLOB": (0xFA, "MEDIUM_BLOB"),
-        "LONG_BLOB": (0xFB, "LONG_BLOB"),
-        "BLOB": (0xFC, "BLOB"),
-        "VAR_STRING": (0xFD, "VAR_STRING"),
-        "STRING": (0xFE, "STRING"),
-        "GEOMETRY": (0xFF, "GEOMETRY"),
+        "DECIMAL": (DECIMAL, "DECIMAL"),
+        "TINY": (TINY, "TINY"),
+        "SHORT": (SHORT, "SHORT"),
+        "LONG": (LONG, "LONG"),
+        "FLOAT": (FLOAT, "FLOAT"),
+        "DOUBLE": (DOUBLE, "DOUBLE"),
+        "NULL": (NULL, "NULL"),
+        "TIMESTAMP": (TIMESTAMP, "TIMESTAMP"),
+        "LONGLONG": (LONGLONG, "LONGLONG"),
+        "INT24": (INT24, "INT24"),
+        "DATE": (DATE, "DATE"),
+        "TIME": (TIME, "TIME"),
+        "DATETIME": (DATETIME, "DATETIME"),
+        "YEAR": (YEAR, "YEAR"),
+        "NEWDATE": (NEWDATE, "NEWDATE"),
+        "VARCHAR": (VARCHAR, "VARCHAR"),
+        "BIT": (BIT, "BIT"),
+        "VECTOR": (VECTOR, "VECTOR"),
+        "JSON": (JSON, "JSON"),
+        "NEWDECIMAL": (NEWDECIMAL, "NEWDECIMAL"),
+        "ENUM": (ENUM, "ENUM"),
+        "SET": (SET, "SET"),
+        "TINY_BLOB": (TINY_BLOB, "TINY_BLOB"),
+        "MEDIUM_BLOB": (MEDIUM_BLOB, "MEDIUM_BLOB"),
+        "LONG_BLOB": (LONG_BLOB, "LONG_BLOB"),
+        "BLOB": (BLOB, "BLOB"),
+        "VAR_STRING": (VAR_STRING, "VAR_STRING"),
+        "STRING": (STRING, "STRING"),
+        "GEOMETRY": (GEOMETRY, "GEOMETRY"),
     }
 
     @classmethod
@@ -444,9 +484,22 @@ class ServerCmd(_Constants, metaclass=ServerCmdMeta):
 
 
 class ClientFlag(_Flags):
-    """MySQL Client Flags
+    """MySQL Client Flags.
 
-    Client options as found in the MySQL sources mysql-src/include/mysql_com.h
+    Client options as found in the MySQL sources mysql-src/include/mysql_com.h.
+
+    This class provides constants defining MySQL client flags that can be used
+    when the connection is established to configure the session. The `ClientFlag`
+    class is available when importing mysql.connector.
+
+    The `ClientFlag` class shouldn't be instantiated.
+
+    Examples:
+        ```
+        >>> import mysql.connector
+        >>> mysql.connector.ClientFlag.FOUND_ROWS
+        2
+        ```
     """
 
     LONG_PASSWD: int = 1 << 0
@@ -644,7 +697,6 @@ class RefreshOption(_Constants, metaclass=RefreshOptionMeta):
     TABLES: int = 1 << 2
     HOST: int = 1 << 3
     STATUS: int = 1 << 4
-    THREADS: int = 1 << 5
     REPLICA: int = 1 << 6
 
     desc: Dict[str, Tuple[int, str]] = {
@@ -653,7 +705,6 @@ class RefreshOption(_Constants, metaclass=RefreshOptionMeta):
         "TABLES": (1 << 2, "close all tables"),
         "HOST": (1 << 3, "Flush host cache"),
         "STATUS": (1 << 4, "Flush status variables"),
-        "THREADS": (1 << 5, "Flush thread cache"),
         "REPLICA": (1 << 6, "Reset source info and restart replica thread"),
         "SLAVE": (1 << 6, "Deprecated option; use REPLICA instead."),
     }
@@ -849,6 +900,8 @@ class SQLMode(_Constants):
     are used when setting the SQL_MODE system variable using the MySQL
     SET command.
 
+    The `SQLMode` class shouldn't be instantiated.
+
     See http://dev.mysql.com/doc/refman/5.6/en/server-sql-mode.html
     """
 
@@ -923,62 +976,6 @@ CONN_ATTRS_DN: List[str] = [
     "_connector_version",
 ]
 
-# TLS v1.0 cipher suites IANI to OpenSSL name translation
-TLSV1_CIPHER_SUITES: Dict[str, str] = {
-    "TLS_RSA_WITH_NULL_MD5": "NULL-MD5",
-    "TLS_RSA_WITH_NULL_SHA": "NULL-SHA",
-    "TLS_RSA_WITH_RC4_128_MD5": "RC4-MD5",
-    "TLS_RSA_WITH_RC4_128_SHA": "RC4-SHA",
-    "TLS_RSA_WITH_IDEA_CBC_SHA": "IDEA-CBC-SHA",
-    "TLS_RSA_WITH_3DES_EDE_CBC_SHA": "DES-CBC3-SHA",
-    "TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA": "Not implemented.",
-    "TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA": "Not implemented.",
-    "TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA": "DHE-DSS-DES-CBC3-SHA",
-    "TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA": "DHE-RSA-DES-CBC3-SHA",
-    "TLS_DH_anon_WITH_RC4_128_MD5": "ADH-RC4-MD5",
-    "TLS_DH_anon_WITH_3DES_EDE_CBC_SHA": "ADH-DES-CBC3-SHA",
-    # AES cipher suites from RFC3268, extending TLS v1.0
-    "TLS_RSA_WITH_AES_128_CBC_SHA": "AES128-SHA",
-    "TLS_RSA_WITH_AES_256_CBC_SHA": "AES256-SHA",
-    "TLS_DH_DSS_WITH_AES_128_CBC_SHA": "DH-DSS-AES128-SHA",
-    "TLS_DH_DSS_WITH_AES_256_CBC_SHA": "DH-DSS-AES256-SHA",
-    "TLS_DH_RSA_WITH_AES_128_CBC_SHA": "DH-RSA-AES128-SHA",
-    "TLS_DH_RSA_WITH_AES_256_CBC_SHA": "DH-RSA-AES256-SHA",
-    "TLS_DHE_DSS_WITH_AES_128_CBC_SHA": "DHE-DSS-AES128-SHA",
-    "TLS_DHE_DSS_WITH_AES_256_CBC_SHA": "DHE-DSS-AES256-SHA",
-    "TLS_DHE_RSA_WITH_AES_128_CBC_SHA": "DHE-RSA-AES128-SHA",
-    "TLS_DHE_RSA_WITH_AES_256_CBC_SHA": "DHE-RSA-AES256-SHA",
-    "TLS_DH_anon_WITH_AES_128_CBC_SHA": "ADH-AES128-SHA",
-    "TLS_DH_anon_WITH_AES_256_CBC_SHA": "ADH-AES256-SHA",
-    # Camellia cipher suites from RFC4132, extending TLS v1.0
-    "TLS_RSA_WITH_CAMELLIA_128_CBC_SHA": "CAMELLIA128-SHA",
-    "TLS_RSA_WITH_CAMELLIA_256_CBC_SHA": "CAMELLIA256-SHA",
-    "TLS_DH_DSS_WITH_CAMELLIA_128_CBC_SHA": "DH-DSS-CAMELLIA128-SHA",
-    "TLS_DH_DSS_WITH_CAMELLIA_256_CBC_SHA": "DH-DSS-CAMELLIA256-SHA",
-    "TLS_DH_RSA_WITH_CAMELLIA_128_CBC_SHA": "DH-RSA-CAMELLIA128-SHA",
-    "TLS_DH_RSA_WITH_CAMELLIA_256_CBC_SHA": "DH-RSA-CAMELLIA256-SHA",
-    "TLS_DHE_DSS_WITH_CAMELLIA_128_CBC_SHA": "DHE-DSS-CAMELLIA128-SHA",
-    "TLS_DHE_DSS_WITH_CAMELLIA_256_CBC_SHA": "DHE-DSS-CAMELLIA256-SHA",
-    "TLS_DHE_RSA_WITH_CAMELLIA_128_CBC_SHA": "DHE-RSA-CAMELLIA128-SHA",
-    "TLS_DHE_RSA_WITH_CAMELLIA_256_CBC_SHA": "DHE-RSA-CAMELLIA256-SHA",
-    "TLS_DH_anon_WITH_CAMELLIA_128_CBC_SHA": "ADH-CAMELLIA128-SHA",
-    "TLS_DH_anon_WITH_CAMELLIA_256_CBC_SHA": "ADH-CAMELLIA256-SHA",
-    # SEED cipher suites from RFC4162, extending TLS v1.0
-    "TLS_RSA_WITH_SEED_CBC_SHA": "SEED-SHA",
-    "TLS_DH_DSS_WITH_SEED_CBC_SHA": "DH-DSS-SEED-SHA",
-    "TLS_DH_RSA_WITH_SEED_CBC_SHA": "DH-RSA-SEED-SHA",
-    "TLS_DHE_DSS_WITH_SEED_CBC_SHA": "DHE-DSS-SEED-SHA",
-    "TLS_DHE_RSA_WITH_SEED_CBC_SHA": "DHE-RSA-SEED-SHA",
-    "TLS_DH_anon_WITH_SEED_CBC_SHA": "ADH-SEED-SHA",
-    # GOST cipher suites from draft-chudov-cryptopro-cptls, extending TLS v1.0
-    "TLS_GOSTR341094_WITH_28147_CNT_IMIT": "GOST94-GOST89-GOST89",
-    "TLS_GOSTR341001_WITH_28147_CNT_IMIT": "GOST2001-GOST89-GOST89",
-    "TLS_GOSTR341094_WITH_NULL_GOSTR3411": "GOST94-NULL-GOST94",
-    "TLS_GOSTR341001_WITH_NULL_GOSTR3411": "GOST2001-NULL-GOST94",
-}
-
-# TLS v1.1 cipher suites IANI to OpenSSL name translation
-TLSV1_1_CIPHER_SUITES: Dict[str, str] = TLSV1_CIPHER_SUITES
 
 # TLS v1.2 cipher suites IANI to OpenSSL name translation
 TLSV1_2_CIPHER_SUITES: Dict[str, str] = {
@@ -1128,15 +1125,11 @@ TLSV1_3_CIPHER_SUITES: Dict[str, str] = {
 }
 
 TLS_CIPHER_SUITES: Dict[str, Dict[str, str]] = {
-    "TLSv1": TLSV1_CIPHER_SUITES,
-    "TLSv1.1": TLSV1_1_CIPHER_SUITES,
     "TLSv1.2": TLSV1_2_CIPHER_SUITES,
     "TLSv1.3": TLSV1_3_CIPHER_SUITES,
 }
 
 OPENSSL_CS_NAMES: Dict[str, ValuesView[str]] = {
-    "TLSv1": TLSV1_CIPHER_SUITES.values(),
-    "TLSv1.1": TLSV1_1_CIPHER_SUITES.values(),
     "TLSv1.2": TLSV1_2_CIPHER_SUITES.values(),
     "TLSv1.3": TLSV1_3_CIPHER_SUITES.values(),
 }
