@@ -1,16 +1,16 @@
-# Copyright (c) 2009, 2023, Oracle and/or its affiliates.
+# Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2.0, as
 # published by the Free Software Foundation.
 #
-# This program is also distributed with certain software (including
+# This program is designed to work with certain software (including
 # but not limited to OpenSSL) that is licensed under separate terms,
 # as designated in a particular file or component or in included license
-# documentation.  The authors of MySQL hereby grant you an
+# documentation. The authors of MySQL hereby grant you an
 # additional permission to link the program and your derivative works
-# with the separately licensed software that they have included with
-# MySQL.
+# with the separately licensed software that they have either included with
+# the program or referenced in the documentation.
 #
 # Without limiting anything contained in the foregoing, this file,
 # which is part of MySQL Connector/Python, is also subject to the
@@ -29,6 +29,7 @@
 """Converting MySQL and Python types
 """
 
+import array
 import datetime
 import math
 import struct
@@ -37,7 +38,7 @@ import time
 from decimal import Decimal
 from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union
 
-from .constants import CharacterSet, FieldFlag, FieldType
+from .constants import MYSQL_VECTOR_TYPE_CODE, CharacterSet, FieldFlag, FieldType
 from .custom_types import HexLiteral
 from .types import (
     DescriptionType,
@@ -740,6 +741,23 @@ class MySQLConverter(MySQLConverterBase):
                 return bytes(value)
         return self._string_to_python(value, dsc)
 
+    @staticmethod
+    def _vector_to_python(
+        value: Optional[bytes], desc: Optional[DescriptionType] = None
+    ) -> Optional[array.array]:
+        """
+        Converts a MySQL VECTOR value to a Python array.array type.
+
+        Returns an array of floats if `value` isn't `None`, otherwise `None`.
+        """
+        if value is None or isinstance(value, array.array):
+            return value
+
+        if isinstance(value, (bytes, bytearray)):
+            return array.array(MYSQL_VECTOR_TYPE_CODE, value)
+
+        raise TypeError(f"Got unsupported type {value.__class__.__name__}")
+
     _long_blob_to_python = _blob_to_python
     _medium_blob_to_python = _blob_to_python
     _tiny_blob_to_python = _blob_to_python
